@@ -15,6 +15,7 @@
 */
 
 #include <assert.h>
+#include <errno.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -66,7 +67,7 @@ static const Node INTERNAL_NODE_NULL = { 0, 0 };
 
 struct SerdReaderImpl {
 	void*             handle;
-	void              (*free_handle)(void*);
+	void              (*free_handle)(void* ptr);
 	SerdBaseSink      base_sink;
 	SerdPrefixSink    prefix_sink;
 	SerdStatementSink statement_sink;
@@ -316,20 +317,24 @@ emit_statement(SerdReader* reader, SerdStatementFlags* flags,
 	const SerdNode datatype  = public_node(reader, d);
 	const SerdNode lang      = public_node_from_ref(reader, SERD_LITERAL, l);
 	object.flags = f;
-	bool ret = !reader->statement_sink(reader->handle,
-	                                   *flags,
-	                                   &graph,
-	                                   &subject,
-	                                   &predicate,
-	                                   &object,
-	                                   &datatype,
-	                                   &lang);
+	bool ret = !reader->statement_sink
+		|| !reader->statement_sink(reader->handle,
+		                           *flags,
+		                           &graph,
+		                           &subject,
+		                           &predicate,
+		                           &object,
+		                           &datatype,
+		                           &lang);
 	*flags = (*flags & SERD_ANON_CONT) ? SERD_ANON_CONT : 0;
 	return ret;
 }
 
-static bool read_collection(SerdReader* reader, ReadContext ctx, Node* dest);
-static bool read_predicateObjectList(SerdReader* reader, ReadContext ctx, bool blank);
+static bool
+read_collection(SerdReader* reader, ReadContext ctx, Node* dest);
+
+static bool
+read_predicateObjectList(SerdReader* reader, ReadContext ctx, bool blank);
 
 // [40]	hex	::=	[#x30-#x39] | [#x41-#x46]
 static inline uint8_t
@@ -711,7 +716,7 @@ static inline uchar
 read_nameStartChar(SerdReader* reader, bool required)
 {
 	const uint8_t c = peek_byte(reader);
-	if (c == '_' || is_alpha(c)) {
+	if (c == '_' || is_alpha(c)) {  // TODO: not strictly correct
 		return eat_byte(reader, c);
 	} else {
 		if (required) {
@@ -734,8 +739,7 @@ read_nameChar(SerdReader* reader)
 	case '-': case 0xB7: case '0': case '1': case '2': case '3': case '4':
 	case '5': case '6': case '7': case '8': case '9':
 		return eat_byte(reader, c);
-	default:
-		// TODO: 0x300-0x036F | 0x203F-0x2040
+	default:  // TODO: 0x300-0x036F | 0x203F-0x2040
 		return 0;
 	}
 	return 0;
@@ -1025,7 +1029,7 @@ blank_id(SerdReader* reader)
 static bool
 read_blank(SerdReader* reader, ReadContext ctx, bool subject, Node* dest)
 {
-	const bool was_anon_subject = (*ctx.flags | SERD_ANON_CONT);
+	const bool was_anon_subject = subject && (*ctx.flags | SERD_ANON_CONT);
 	switch (peek_byte(reader)) {
 	case '_':
 		*dest = make_node(SERD_BLANK, read_nodeID(reader));
@@ -1062,7 +1066,7 @@ read_blank(SerdReader* reader, ReadContext ctx, bool subject, Node* dest)
 			const SerdNode end = public_node(reader, dest);
 			reader->end_sink(reader->handle, &end);
 		}
-		if (subject && !was_anon_subject) {
+		if (!was_anon_subject) {
 			*ctx.flags &= ~SERD_ANON_CONT;
 		}
 		return true;
@@ -1327,7 +1331,9 @@ read_base(SerdReader* reader)
 	Ref uri;
 	TRY_RET(uri = read_uriref(reader));
 	const SerdNode uri_node = public_node_from_ref(reader, SERD_URI, uri);
-	reader->base_sink(reader->handle, &uri_node);
+	if (reader->base_sink) {
+		reader->base_sink(reader->handle, &uri_node);
+	}
 	pop_string(reader, uri);
 	return true;
 }
@@ -1340,7 +1346,7 @@ read_prefixID(SerdReader* reader)
 	// `@' is already eaten in read_directive
 	eat_string(reader, "prefix", 6);
 	TRY_RET(read_ws_plus(reader));
-	bool ret = false;
+	bool ret = true;
 	Ref name = read_prefixName(reader);
 	if (!name) {
 		name = push_string(reader, "", 0);
@@ -1351,7 +1357,9 @@ read_prefixID(SerdReader* reader)
 	TRY_THROW(uri = read_uriref(reader));
 	const SerdNode name_node = public_node_from_ref(reader, SERD_LITERAL, name);
 	const SerdNode uri_node  = public_node_from_ref(reader, SERD_URI, uri);
-	ret = !reader->prefix_sink(reader->handle, &name_node, &uri_node);
+	if (reader->prefix_sink) {
+		ret = !reader->prefix_sink(reader->handle, &name_node, &uri_node);
+	}
 	pop_string(reader, uri);
 except:
 	pop_string(reader, name);
@@ -1504,6 +1512,7 @@ serd_reader_read_file(SerdReader*    reader,
 
 	FILE* fd = fopen((const char*)path, "r");
 	if (!fd) {
+		fprintf(stderr, "Error opening file %s (%s)\n", path, strerror(errno));
 		return SERD_ERR_UNKNOWN;
 	}
 	SerdStatus ret = serd_reader_read_file_handle(reader, fd, path);
